/* ----------------------------------------------------------------------------
 * scroll version 0.1
 * emulates mouse wheel
 *
 * Copyright 2004 Takuo Yonezawa (takuo@aya.or.jp)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * ----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <linux/input.h>

static void write_event(int type, int code, int value)
{
	struct input_event event;

	event.type = type;
	event.code = code;
	event.value = value;
	gettimeofday(&event.time, NULL);
	if (write(STDOUT_FILENO, &event, sizeof(event)) < sizeof(event)) {
		perror("write_event");
  }
}

static void report_scroll(int dy)
{
  write_event(EV_REL, REL_WHEEL, -dy);
}

static void report_key(int key)
{
  write_event(EV_KEY, key, 1);
  write_event(EV_KEY, key, 0);
}

static void event_loop(int wheel_threshold)
{
  struct input_event events[64];
  size_t read_size;
  int current_y = 0;
  int i;
  
  for (;;) {  
    read_size = read(STDIN_FILENO, events, sizeof(events));
    for (i = 0; i < (int)(read_size / sizeof(struct input_event)); i++){
      switch(events[i].type) {
        case EV_REL:
          switch(events[i].code) {
            case REL_Y:
              if ((int)events[i].value < 0) {
                if (current_y > 0) {
                  current_y = (int)events[i].value;
                } else {
                  current_y += (int)events[i].value;
                }

                if (current_y < -wheel_threshold) {
                  report_scroll(1);
                  current_y += wheel_threshold;
                }
              } else {
                if (current_y < 0) {
                  current_y = (int)events[i].value;
                } else {
                  current_y += (int)events[i].value;
                }
                
                if (current_y > wheel_threshold) {
                  report_scroll(-1);
                  current_y -= wheel_threshold;
                }
              }
              break;
            default:
              break;
          }
        case EV_KEY:
          switch(events[i].code) {
            case BTN_LEFT:
              write_event(EV_KEY, KEY_END, events[i].value);
              break;
            case BTN_RIGHT:
              write_event(EV_KEY, KEY_HOME, events[i].value);
              break;
            case BTN_MIDDLE:
              if (events[i].value == 0) {
                write_event(EV_KEY, KEY_LEFTCTRL, 1);
                write_event(EV_KEY, KEY_W, 1);
                write_event(EV_KEY, KEY_W, 0);
                write_event(EV_KEY, KEY_LEFTCTRL, 0);
              }
            default:
              break;
          }
          break;
        default:
          break;
      }
    }
  }
}

int main(void)
{
  int wheel_threshold = 10;

  event_loop(wheel_threshold);

  exit(EXIT_SUCCESS);
}
