/* Դ */

#include <sys/time.h>
#include <stdlib.h>
#include <stdint.h>

typedef enum _ImPS2DeviceID ImPS2DeviceID;
enum _ImPS2DeviceID {
  IMPS2_PS2_MOUSE = 0x0,
  IMPS2_INTELLI_MOUSE = 0x3,
};

typedef enum _ImPS2Mode ImPS2Mode;
enum _ImPS2Mode {
  IMPS2_STREAM_MODE,
  IMPS2_REMOTE_MODE,
  IMPS2_WRAP_MODE,
};

enum ImPS2ButtonBitsMask {
  IMPS2_LEFT_BUTTON = 0x1,
  IMPS2_RIGHT_BUTTON = 0x2,
  IMPS2_MIDDLE_BUTTON = 0x4,
};

typedef enum _ImPS2Scaling ImPS2Scaling;
enum _ImPS2Scaling {
  IMPS2_SCALING_1_TO_1,
  IMPS2_SCALING_2_TO_1,
}

typedef struct _ImPS2Mouse ImPS2Mouse;
struct _ImPS2Mouse {
  int counter_x;
  int counter_y;
  int counter_z;
  int button_bits;
  ImPS2DeviceID device_id;
  ImPS2Mode mode;
  ImPS2Mode previous_mode_of_wrap_mode;
  int sample_rate;
  int resolution;
  ImPS2Scaling scaling;
  int data_reporting_enabled;
  int8_t last_packet[4];
  int last_packet_size;
  struct timeval last_sent_time;
};

static int scaling_table[] = {0, 1, 1, 3, 6, 9};

void imps2_move_relative(ImPS2Mouse *mouse, File* output, int rx, int ry)
{
  mouse->counter_x += mouse->resolution * rx;
  mouse->counter_y += mouse->resolution * ry;
  imps2_tick(ImPS2Mouse *mouse, File* output);
}

void imps2_set_button(ImPS2Mouse *mouse, File* output, int button_bits)
{
  mouse->button_bits = button_bits & (IMPS2_LEFT_BUTTON | IMPS2_RIGHT_BUTTON | IMPS2_MIDDLE_BUTTON);
  imps2_tick(ImPS2Mouse *mouse, File* output);
}

void imps2_tick(ImPS2Mouse *mouse, File* output) 
{
  struct timeval now;
  struct timeval time_since_last_sent;

  if (!mouse->data_reporting_enabled) {
    return;
  }

  gettimeofday(&now);
  time_since_last_sent.tv_sec = now.tv_sec - mouse->last_sent_time.tv_sec;
  if (now.tv_usec > mouse->last_sent_time.tv_usec) {
    time_since_last_sent.tv_usec = now->tv_usec - mouse->last_sent_time.tv_usec;
  } else {
    time_since_last_sent.tv_usec = now.tv_usec - time_since_last_sent.tv_usec;
    time_since_last_sent.tv_sec -= 1;
  }

  if (time_since_last_sent.tv_sec >= 1 ||
      time_since_last_sent.tv_usec >= 1000000 / mouse->sample_rate) {
    imps2_send_packet(mouse, output, now, 1);
  }
}

void imps2_request(ImPS2Mouse mouse, File* outupt)
{
}

static void imps2_send_packet(ImPS2Mouse mouse, File* outupt, struct timeval now, int enable_scaling)
{
  int clipped_x;
  int clipped_y;
  int clipped_z = mouse->counter_z > 7 ? 7 : mouse->counter_z < -8 ? -8 : mouse->counter_z;

  switch (enable_scaling ? mouse->scaling : IMPS2_SCALING_1_TO_1) {
    case IMPS2_SCALING_1_TO_1:
      clipped_x = mouse->counter_x;
      clipped_y = mouse->counter_y;
      break;
    case IMPS2_SCALING_2_TO_1:
      if (abs(mouse->counter_x) > 5) {
        clipped_x = mouse->counter_x * 2;
      } else if (mouse->counter_x >= 0) {
        clipped_x = scaling_table[mouse->counter_x];
      } else {
        clipped_x = -scaling_table[-mouse->counter_x]
      }
      break;
    default:
      break;
  }
  clipped_x = clipped_x > 255 ? 255 : clipped_x < -255 ? -255 : clipped_x;
  clipped_y = clipped_y > 255 ? 255 : clipped_y < -255 ? -255 : clipped_y;

  mouse->last_packet[0] = ((mouse->counter_y < 0) << 5) | ((mouse->counter_x < 0) << 4) | (0x1 << 3) | mouse->button_bits;
  mouse->last_packet[1] = clipped_x & 0xFF;
  mouse->last_packet[2] = clipped_y & 0xFF;
  mouse->counter_x -= clipped_x;
  mouse->counter_y -= clipped_y;
  mouse->last_sent_time = now;

  switch (mouse->device_id) {
    case IMPS2_PS2_MOUSE:
      mouse->last_packet_size = 3;      
      break;
    case IMPS2_INTELLI_MOUSE:
      mouse->last_packet_size = 4;
      mouse->last_packet[3] = clipped_z;
      mouse->counter_z -= clipped_z;
      break;
    default:
      mouse->last_packet_size = 0;
      break;
  }

  fwrite(mouse->last_packet, mouse->last_packet_size, 1, output);
}
