/* ----------------------------------------------------------------------------
 * scroll version 0.1
 * emulates mouse wheel
 *
 * Copyright 2004 Takuo Yonezawa (takuo@aya.or.jp)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * ----------------------------------------------------------------------------*/

#include <linux/input.h>
#include <sys/ioctl.h>
#include "linux/uinput.h"
#include "main.h"

char *name = "scroll";
char *version_information =
  "scroll version 0.1\n"
  "\n"
  "Copyright (C) 2004 Takuo Yonezawa (takuo@aya.or.jp)\n"
  "\n"
  "This is free software; see the source for copying conditions.\n"
  "There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n";
char *description = "Convert mouse motion from device to mouse wheel scroll.\n";
#define WHEEL_THRESHOLD 10

static int current_y;

void *init_event_handler(int number_of_event_device_files, int *event_fds)
{
  current_y = 0;
  return (void *)&current_y;
}

void finalize_event_hander(void *user_data)
{
}

void handle_event(int fd, struct input_event event, void *user_data)
{
  int *current_y = (int *)user_data;

  switch(event.type) {
    case EV_REL:
      switch(event.code) {
        case REL_Y:
          if ((int)event.value < 0) {
            if (*current_y > 0) {
              *current_y = (int)event.value;
            } else {
              *current_y += (int)event.value;
            }

            if (*current_y < -WHEEL_THRESHOLD) {
              report_scroll(-1);
              *current_y += WHEEL_THRESHOLD;
            }
          } else {
            if (*current_y < 0) {
              *current_y = (int)event.value;
            } else {
              *current_y += (int)event.value;
            }
                
            if (*current_y > WHEEL_THRESHOLD) {
              report_scroll(1);
              *current_y -= WHEEL_THRESHOLD;
            }
          }
          break;
        default:
          break;
      }
      break;
    case EV_KEY:
      switch(event.code) {
        case BTN_LEFT:
          send_event(EV_KEY, KEY_END, event.value);
          break;
        case BTN_RIGHT:
          send_event(EV_KEY, KEY_HOME, event.value);
          break;
        case BTN_MIDDLE:
          if (event.value == 0) {
            send_event(EV_KEY, KEY_LEFTCTRL, 1);
            send_event(EV_KEY, KEY_W, 1);
            send_event(EV_KEY, KEY_W, 0);
            send_event(EV_KEY, KEY_LEFTCTRL, 0);
          }
        default:
          break;
      }
      break;
    default:
      break;
  }
}

void set_event_bits(int ui_fd)
{  
	ioctl(ui_fd, UI_SET_EVBIT, EV_REL);
  ioctl(ui_fd, UI_SET_RELBIT, REL_WHEEL);
  ioctl(ui_fd, UI_SET_RELBIT, REL_X);
  ioctl(ui_fd, UI_SET_RELBIT, REL_Y);

  ioctl(ui_fd, UI_SET_EVBIT, EV_KEY);
  ioctl(ui_fd, UI_SET_KEYBIT, KEY_LEFTCTRL);
  ioctl(ui_fd, UI_SET_KEYBIT, KEY_HOME);
  ioctl(ui_fd, UI_SET_KEYBIT, KEY_END);
  ioctl(ui_fd, UI_SET_KEYBIT, KEY_W);
  /* to make mousedev to recognize the device as mouse */
  ioctl(ui_fd, UI_SET_KEYBIT, BTN_LEFT);
}
