/* ----------------------------------------------------------------------------
 * toggle_scroll version 0.1
 * convert cursor movement to cursor movement or wheel
 *
 * Copyright 2004 Takuo Yonezawa (takuo@aya.or.jp)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * ----------------------------------------------------------------------------*/

#include <stdio.h>
#include <linux/input.h>
#include <sys/ioctl.h>
#include "linux/uinput.h"
#include "main.h"

char *name = "toggle_scroll";
char *version_information =
  "toggle_scroll 0.1\n"
  "\n"
  "Copyright (C) 2004 Takuo Yonezawa (takuo@aya.or.jp)\n"
  "\n"
  "This is free software; see the source for copying conditions.\n"
  "There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n";
char *description = "Convert cursor movement to cursor movement or wheel.\n";
#define WHEEL_THRESHOLD 20

typedef enum _ConvertType ConvertType;
enum _ConvertType {
  CONVERT_MODE_CURSOR,
  CONVERT_MODE_WHEEL,
};

typedef struct _ConvertState ConvertState;
struct _ConvertState {
  int current_y;
  int mouse_fd;
  ConvertType mode;
};

static ConvertState state;

void *init_event_handler(int number_of_event_device_files, int *event_fds)
{
  state.current_y = 0;
  state.mode = CONVERT_MODE_CURSOR;
  state.mouse_fd = event_fds[0];
  return (void *)&state;
}

void finalize_event_hander(void *user_data)
{
}

void handle_event(int fd, struct input_event event, void *user_data)
{
  ConvertState *state = (ConvertState *)user_data;

  switch(event.type) {
    case EV_REL:
      switch(event.code) {
        case REL_Y:
          if (state->mode == CONVERT_MODE_CURSOR) {
            send_event(event.type, event.code, event.value);
          } else if (state->mode == CONVERT_MODE_WHEEL) {
            if ((int)event.value < 0) {
              if (state->current_y > 0) {
                state->current_y = (int)event.value;
              } else {
                state->current_y += (int)event.value;
              }

              if (state->current_y < -WHEEL_THRESHOLD) {
                report_scroll(-1);
                state->current_y += WHEEL_THRESHOLD;
              }
            } else {
              if (state->current_y < 0) {
                state->current_y = (int)event.value;
              } else {
                state->current_y += (int)event.value;
              }
                
              if (state->current_y > WHEEL_THRESHOLD) {
                report_scroll(1);
                state->current_y -= WHEEL_THRESHOLD;
              }
            }
          }
          break;
        default:
          if (state->mode == CONVERT_MODE_CURSOR) {
            send_event(event.type, event.code, event.value);
          }
          break;
      }
      break;
    case EV_KEY:
      switch(event.code) {
        case KEY_INTL5:
          if (event.value == 1) {
            state->mode = CONVERT_MODE_WHEEL;
          } else if (event.value == 0) {
            state->mode = CONVERT_MODE_CURSOR;
            state->current_y = 0;
          }
          break;
        case BTN_LEFT:
          if (state->mode == CONVERT_MODE_WHEEL) {
            send_event(EV_KEY, KEY_HOME, event.value);
          } else {
            send_event(event.type, event.code, event.value);
          }
          break;
        case BTN_RIGHT:
          if (state->mode == CONVERT_MODE_WHEEL) {
            send_event(EV_KEY, KEY_END, event.value);
          } else {
            send_event(event.type, event.code, event.value);
          }
          break;
        default:
          if (fd == state->mouse_fd) {
            send_event(event.type, event.code, event.value);
          }
          break;
      }
      break;
    default:
      break;
  }
}

void set_event_bits(int ui_fd)
{  
	ioctl(ui_fd, UI_SET_EVBIT, EV_REL);
  ioctl(ui_fd, UI_SET_RELBIT, REL_WHEEL);
  ioctl(ui_fd, UI_SET_RELBIT, REL_X);
  ioctl(ui_fd, UI_SET_RELBIT, REL_Y);

  ioctl(ui_fd, UI_SET_EVBIT, EV_KEY);
  ioctl(ui_fd, UI_SET_KEYBIT, KEY_HOME);
  ioctl(ui_fd, UI_SET_KEYBIT, KEY_END);
  ioctl(ui_fd, UI_SET_KEYBIT, BTN_LEFT);
  ioctl(ui_fd, UI_SET_KEYBIT, BTN_RIGHT);
  ioctl(ui_fd, UI_SET_KEYBIT, BTN_MIDDLE);
}
